-- The Macaulay 2 code for Daniel Barath, Zuzana Kukelova, Relative Pose from SIFT Features, ECCV 2022
-- The code generates the contraint relating epipolar geometry and SIFT
-- The code can be run online on:
-- https://www.unimelb-macaulay2.cloud.edu.au/#home

R = QQ[a1,a2,a3,a4,              -- affinity   A
       c1,s1,c2,s2,q1,q2,        -- rotations(sin,cos)+scales (SIFT)
       q,                         -- q2^2/q1^2
       w,                        -- shear
       f1,f2,f3,f4,f5,f6,f7,f8,f9,   -- elements of F
       u1,u2,v1,v2                  -- point coreespondences
      ]       

-- Constraints relating elements of  A to the scales and rotations of two features in two images 
h1 = c1 * q1 * a1 + s1 * q1 * a2 - c2 * q2
h2 = c1 * q1 * a3 + s1 * q1 * a4 - s2 * q2
h3 = a1*a4-a2*a3-q^2

-- Constraint from scales
h4 = q1*q-q2

-- Constraints from sin and cos
h5 = c1^2+s1^2-1
h6 = c2^2+s2^2-1

-- Ideal
I = ideal(h1|h2|h3|h4|h5|h6)

S = ideal(q1)

I1 = saturate(I,S)

-- Elimination ideal
I2 = eliminate({q1,q2},I1)

p = mingens I2

--Constraints on F
q1 = (u2 + a1*u1)*f1 + a1*v1*f2 + a1*f3 + (v2 + a3*u1)*f4 + a3*v1*f5 + a3*f6 + f7
q2 =  a2*u1*f1 + (u2 + a2*v1)*f2 + a2*f3 + a4*u1*f4 + (v2 + a4*v1)*f5 + a4*f6 + f8

-- Ideal
J = ideal(p_2_0| p_3_0| p_4_0| q1|q2)
 
-- Elimination ideal
J1 = eliminate({a1,a2,a3,a4},J)

-- Constraint relating epipolar geometry and SIFT
mingens J1
